from __future__ import absolute_import
from __future__ import print_function
from zaguan.engines import WebKitMethods, QTWebKitMethods
from zaguan.functions import asynchronous_gtk_message


def launch_browser(uri, echo=False, user_settings=None, qt=False, window=None):
    """Creates and initialize a browser object"""
    if qt:
        implementation = QTWebKitMethods
    else:
        implementation = WebKitMethods

    browser = implementation.create_browser()
    implementation.set_settings(browser, user_settings)

    implementation.open_uri(browser, uri)

    def web_send(msg):
        if echo:
            print('<<<', msg)
        asynchronous_gtk_message(implementation.inject_javascript)(browser,
                                                                   msg)

    return browser, web_send



from __future__ import absolute_import

try:
    import gobject
    idle_add = gobject.idle_add
except ImportError:
    try:
        from gi.repository import GLib
        idle_add = GLib.idle_add
    except ImportError:
        pass

from zaguan.constants import WEBKIT


def get_implementation():
    try:
        import webkit
        have_webkit = True
    except:
        have_webkit = False

    if have_webkit:
        implementation = WEBKIT
    else:
        raise Exception('Failed to import webkit module')
    return implementation


def asynchronous_gtk_message(fun):
    def worker(xxx_todo_changeme):
        (function, args, kwargs) = xxx_todo_changeme
        function(*args, **kwargs)

    def fun2(*args, **kwargs):
        idle_add(worker, (fun, args, kwargs))

    return fun2



from __future__ import absolute_import
# Copyright (C) 2008 Jan Alonzo <jmalonzo@unpluggable.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

try:
    import gtk
    import webkit
except ImportError:
    from gi.repository import Gtk as gtk, WebKit as webkit


class Inspector(gtk.Window):
    def __init__ (self, inspector):
        """initialize the WebInspector class"""
        gtk.Window.__init__(self)
        self._web_inspector = inspector

        self._web_inspector.connect("inspect-web-view",
                                    self._inspect_web_view_cb)
        self._web_inspector.connect("show-window",
                                    self._show_window_cb)
        self._web_inspector.connect("attach-window",
                                    self._attach_window_cb)
        self._web_inspector.connect("detach-window",
                                    self._detach_window_cb)
        self._web_inspector.connect("close-window",
                                    self._close_window_cb)
        self._web_inspector.connect("finished",
                                    self._finished_cb)

        self.connect("delete-event", self._close_window_cb)

    def _inspect_web_view_cb (self, inspector, web_view):
        """Called when the 'inspect' menu item is activated"""
        self.show()
        scrolled_window = gtk.ScrolledWindow()
        webview = webkit.WebView()
        scrolled_window.set_size_request(width=800, height=600)
        scrolled_window.add(webview)
        scrolled_window.show_all()

        self.add(scrolled_window)
        return webview

    def _show_window_cb (self, inspector):
        """Called when the inspector window should be displayed"""
        self.present()
        return True

    def _attach_window_cb (self, inspector):
        """Called when the inspector should displayed in the same
        window as the WebView being inspected
        """
        return False

    def _detach_window_cb (self, inspector):
        """Called when the inspector should appear in a separate window"""
        return False

    def _close_window_cb (self, inspector, web_view=None):
        """Called when the inspector window should be closed"""
        self.hide()
        return True

    def _finished_cb (self, inspector):
        """Called when inspection is done"""
        self._web_inspector = 0
        self.destroy()
        return False





from __future__ import absolute_import
from __future__ import print_function
try:
    import gtk
    import gobject
    from gtk import gdk
    timeout_add = gobject.timeout_add
    WINDOW_TOPLEVEL = gtk.WINDOW_TOPLEVEL
    WIN_POS_CENTER_ALWAYS = gtk.WIN_POS_CENTER_ALWAYS
except ImportError:
    print("GTK not installed")
    try:
        from gi.repository import GLib, Gtk as gtk, Gdk as gdk
        timeout_add = GLib.timeout_add
        WINDOW_TOPLEVEL = gtk.WindowType.TOPLEVEL
        WIN_POS_CENTER_ALWAYS = gtk.WindowPosition.CENTER_ALWAYS
    except ImportError:
        print("PyGi not installed")
try:
    import sys
    from PyQt4.QtCore import *
    from PyQt4.QtGui import *
except ImportError:
    print("QT not installed")


from time import sleep

from zaguan.controller import WebContainerController


class Zaguan(object):
    def __init__(self, uri, controller=None):
        if controller is None:
            controller = WebContainerController()
        self.controller = controller
        self.uri = uri
        self.on_close = None
        self.exiting = False

    def run(self, settings=None, window=None, debug=False,
            qt=False, on_close=None, cef=False):
        self.on_close = on_close

        if qt:
            self.run_qt(settings, window, debug)
        elif cef:
            self.run_cef(settings, window, debug)
        else:
            self.run_gtk(settings, window, debug)

    def run_cef(self, settings=None, window=None, debug=False):
        gdk.threads_init()

        if window is None:
            self.window = gtk.Window(WINDOW_TOPLEVEL)
            self.window.set_position(WIN_POS_CENTER_ALWAYS)
            self.window.set_size_request(width=800, height=600)
            self.window.realize()
        else:
            self.window = window

        self.window.set_border_width(0)

        self.controller.get_cef_browser(self.uri, self.window)
        sleep(1)
        self.window.show_all()
        self.window.show()
        timeout_add(10, self.OnTimer)

        gtk.main()
        cefpython.Shutdown()

    def OnTimer(self):
        if self.exiting:
            return False
        from cefpython3 import cefpython
        cefpython.MessageLoopWork()
        return True

    def OnExit(self, widget, data=None):
        self.exiting = True
        gtk.main_quit()

    def run_gtk(self, settings=None, window=None, debug=False):
        gdk.threads_init()

        if window is None:
            self.window = gtk.Window(WINDOW_TOPLEVEL)
            self.window.set_position(WIN_POS_CENTER_ALWAYS)
        else:
            self.window = window

        browser = self.controller.get_browser(self.uri, debug=debug,
                                              settings=settings)
        self.window.connect("delete-event", self.quit)
        self.window.set_border_width(0)
        self.window.add(browser)

        sleep(1)
        self.window.show_all()
        self.window.show()
        gtk.main()

    def run_qt(self, settings, window, debug):
        if window is None:
            self.window = QApplication(sys.argv)
        else:
            self.window = window

        browser = self.controller.get_browser(self.uri, debug=debug,
                                              settings=settings, qt=True)
        browser.show()
        sys.exit(self.window.exec_())

    def quit(self, widget, event):
        if self.on_close is not None:
            self.on_close(widget, event)



# -*- coding: utf-8 -*-
from __future__ import absolute_import

try:
    from urllib.parse import unquote
except ImportError:
    from urllib import unquote


from json import dumps, loads

from zaguan.container import launch_browser
from zaguan.inspector import Inspector



class WebContainerController(object):
    """Base class for the web containers of the controllers."""
    def __init__(self):
        self.processors = []

    def on_navigation_requested(self, elem, view, frame, req, data=None):
        """This is the Callback executed each time  URI inside the webkit
        object is loaded"""
        uri = req.get_uri()
        self.process_uri(uri)

    def process_uri(self, uri):
        """Procces the url on each processor."""
        for processor in self.processors:
            processor(uri)

    def set_screen(self, screen, **kwargs):
        """Sends the change sreen command to the web interface."""
        self.send_command("change_screen", [screen, kwargs])

    def send_command(self, command, data=None):
        """Inyects the JS in the browser for the givven command."""
        json_data = dumps(data).replace("\\\"", "\\\'")
        self.send_function("run_op('%s', '%s')" % (command, json_data))

    def get_browser(self, uri, settings=None, debug=False, qt=False):
        """Gets the browser objects and prpare it to bo able to be used in it's
        context.
        """
        if settings is None:
            settings = []

        if debug:
            settings.append(('enable-default-context-menu', True))
            settings.append(('enable-developer-extras', True))

        browser, web_send = launch_browser(uri, echo=debug,
                                           user_settings=settings, qt=qt)
        if debug:
            try:
                inspector = browser.get_web_inspector()
            except AttributeError:
                inspector = browser.get_inspector()

            self.inspector = Inspector(inspector)

        self.send_function = web_send
        if not qt:
            browser.connect("resource-request-starting",
                            self.on_navigation_requested)
        return browser

    def get_cef_browser(self, uri, window, debug=False):
        import re
        from cefpython3 import cefpython

        settings = {
            "debug": False, #debug, # cefpython debug messages in console and in log_file
            "log_severity": cefpython.LOGSEVERITY_INFO, # LOGSEVERITY_VERBOSE
            "release_dcheck_enabled": True, # Enable only when debugging
            # This directories must be set on Linux
            "locales_dir_path": cefpython.GetModuleDirectory()+"/locales",
            "resources_dir_path": cefpython.GetModuleDirectory(),
            "browser_subprocess_path": "%s/%s" % (
                cefpython.GetModuleDirectory(), "subprocess"),
        }

        cefpython.Initialize(settings)

        m = re.search("GtkWindow at 0x(\w+)", str(window))
        hexID = m.group(1)
        windowID = int(hexID, 16)

        windowInfo = cefpython.WindowInfo()
        windowInfo.SetAsChild(windowID)
        browser = cefpython.CreateBrowserSync(
            windowInfo,
            browserSettings={
                "web_security_disabled": True,
            },
            navigateUrl=uri)
        self.cef_window = browser

        jsBindings = cefpython.JavascriptBindings(
            bindToFrames=False, bindToPopups=True)

        jsBindings.SetFunction("zaguan", self.process_uri)
        browser.SetJavascriptBindings(jsBindings)

        self.send_function = browser.GetMainFrame().ExecuteJavascript


    def add_processor(self, url_word, instance=None):
        def _inner(uri):
            scheme, path = uri.split(':', 1)
            if scheme == "http":
                parts = path.split("/")[2:]
                if parts[0] == url_word:
                    remain = parts[1]
                elif parts[1] == url_word:
                    remain = parts[2]
                else:
                    remain = None
                if remain is not None:
                    try:
                        action, data = remain.split("?")
                    except ValueError:
                        action = remain
                        data = "null"

                    data = loads(unquote(data))
                    # search the action at the 'action controller' instance
                    # argument. if we dont find the action, we try to get it
                    # from the controller itself.
                    method = getattr(instance, action, None)
                    if method is None:
                        method = getattr(self, action, None)

                    if not method:
                        raise NotImplementedError(action)
                    return method(data)

        self.processors.append(_inner)
